# -*- coding: utf-8 -*-


class GOSTaskException(Exception):
    pass


class GOSExecutableContainerException(Exception):
    pass


class GOSIOException(Exception):
    pass


class GOSCriticalException(Exception):
    pass



# -*- coding: utf-8 -*-
import unittest

import importlib
import os
import tempfile
from gos.exceptions import GOSTaskException, GOSIOException
from gos.tasks import BaseTask, TaskLoader


class BaseTaskTestCase(unittest.TestCase):
    def setUp(self):
        self.task_class = BaseTask

    def test_name_property(self):
        task = self.task_class()
        self.assertTrue(hasattr(task, "name"))

    def test_self_loop_property(self):
        task = self.task_class()
        self.assertTrue(hasattr(task, "self_loop"))

    def test_do_self_loop_property(self):
        task = self.task_class()
        self.assertTrue(hasattr(task, "do_self_loop"))

    def test_run_method_existence(self):
        task = self.task_class()
        self.assertTrue(hasattr(task, "run"))
        self.assertTrue(callable(getattr(task, "run")))


class TaskLoaderTestCase(unittest.TestCase):
    def test_load_from_file_file_does_not_exists(self):
        file_path = "non_existing_file_pass"
        with self.assertRaises(GOSIOException):
            TaskLoader().load_tasks_from_file(file_path)

    def test_load_from_file_dir_supplied(self):
        dir_path = os.path.dirname(__file__)
        with self.assertRaises(GOSIOException):
            TaskLoader().load_tasks_from_file(dir_path)

    def test_load_from_file_no_name_attribute_on_loaded_class(self):
        bad_class_code = """class MyTaskOne(BaseTask):\n\tdef run(self, assembler_manager):\n\t\tpass\n"""
        source_file = self.create_tmp_py_file()
        source_file.write(self.get_base_task_import_code_string())
        source_file.write(bad_class_code)
        source_file.flush()
        importlib.invalidate_caches()  # importlib invalidate_caches call is required due to this python issue: http://bugs.python.org/issue23412
        source_file_name = source_file.name
        with self.assertRaises(GOSTaskException):
            TaskLoader().load_tasks_from_file(source_file_name)
        source_file.close()

    def test_load_from_file_non_python_file(self):
        tmp_file = tempfile.NamedTemporaryFile(mode="wt", suffix=".non_py")
        importlib.invalidate_caches()
        with self.assertRaises(GOSIOException):
            TaskLoader().load_tasks_from_file(tmp_file.name)

    def get_base_task_import_code_string(self):
        return "from gos.tasks import BaseTask\n"

    @staticmethod
    def get_tasks_names():
        return ["my_task_one",
                "my_task_two",
                "my_task_three",
                "my_task_four",
                "my_task_five",
                "my_task_six",
                "my_task_seven"]

    def get_custom_task_files_values(self):
        return [
            """class MyTaskOne(BaseTask):\n\tname = "my_task_one"\n\tdef run(self, assembler_manager):\n\t\tpass\n""",
            """class MyTaskTwo(BaseTask):\n\tname = "my_task_two"\n\tdef run(self, assembler_manager):\n\t\tpass\n""",
            """class MyTaskThree(BaseTask):\n\tname = "my_task_three"\n\tdef run(self, assembler_manager):\n\t\tpass\n""",
            """class MyTaskFour(BaseTask):\n\tname = "my_task_four"\n\tdef run(self, assembler_manager):\n\t\tpass\n""",
            """class MyTaskFive(BaseTask):\n\tname = "my_task_five"\n\tdef run(self, assembler_manager):\n\t\tpass\n""",
            """class MyTaskSix(BaseTask):\n\tname = "my_task_six"\n\tdef run(self, assembler_manager):\n\t\tpass\n""",
            """class MyTaskSeven(BaseTask):\n\tname = "my_task_seven"\n\tdef run(self, assembler_manager):\n\t\tpass\n""",
        ]

    def get_custom_non_task_classes_code_strings(self):
        return [
            """class MyNoneTaskClass(object):\n\tname = "my_none_base_task"\n\tdef run(self, assembler_manager):\n\t\tpass\n""",
        ]

    def test_load_from_file_single_custom_task_class(self):
        with tempfile.NamedTemporaryFile(mode="wt", suffix=".py") as source_file:
            custom_task_file_data = self.get_custom_task_files_values()[0]
            source_file.write(self.get_base_task_import_code_string())
            source_file.write(custom_task_file_data)
            source_file.flush()
            importlib.invalidate_caches()
            source_file_name = source_file.name
            result = TaskLoader().load_tasks_from_file(source_file_name)
            self.assertIn("my_task_one", result)
            self.assertTrue(issubclass(result["my_task_one"], BaseTask))

    def create_tmp_py_file(self):
        tmp_file = tempfile.NamedTemporaryFile(mode="wt", suffix=".py")
        return tmp_file

    def test_load_from_file_multiple_custom_tasks_classes(self):
        source_file = self.create_tmp_py_file()
        source_file.write(self.get_base_task_import_code_string())
        for custom_task_code_string in self.get_custom_task_files_values():
            source_file.write(custom_task_code_string)
        source_file.write(self.get_custom_non_task_classes_code_strings()[0])
        source_file.flush()
        importlib.invalidate_caches()
        source_file_name = source_file.name
        result = TaskLoader().load_tasks_from_file(source_file_name)
        for task_name in ["my_task_one", "my_task_two", "my_task_three"]:
            self.assertIn(task_name, result)
            self.assertTrue(issubclass(result[task_name], BaseTask))
        self.assertNotIn("my_non_base_task", result)
        source_file.close()

    def test_load_from_dir_dir_does_not_exists(self):
        non_existing_dir = "/my_non/existing_dir"
        with self.assertRaises(GOSTaskException):
            TaskLoader().load_tasks_from_dir(non_existing_dir)

    def test_load_from_dir_file_supplied(self):
        tmp_file = tempfile.NamedTemporaryFile(mode="wt")
        with self.assertRaises(GOSTaskException):
            TaskLoader().load_tasks_from_dir(tmp_file.name)

    def test_load_from_dir_single_file_with_multiple_classes(self):
        tmp_dir = tempfile.TemporaryDirectory()
        tmp_file1 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir.name, delete=False)
        tmp_file1.write(self.get_base_task_import_code_string())
        for task_class_string in self.get_custom_task_files_values():
            tmp_file1.write(task_class_string)
        tmp_file1.flush()
        importlib.invalidate_caches()
        result = TaskLoader().load_tasks_from_dir(tmp_dir.name)
        for task_name in ["my_task_one", "my_task_two", "my_task_three"]:
            self.assertIn(task_name, result)
            self.assertTrue(issubclass(result[task_name], BaseTask))
        tmp_dir.cleanup()

    def test_load_from_dir_non_python_file(self):
        tmp_dir = tempfile.TemporaryDirectory()
        tmp_file1 = tempfile.NamedTemporaryFile(mode="wt", suffix=".non_py", dir=tmp_dir.name, delete=False)
        tmp_file1.write(self.get_base_task_import_code_string())
        for task_class_string in self.get_custom_task_files_values():
            tmp_file1.write(task_class_string)
        tmp_file1.flush()
        importlib.invalidate_caches()
        result = TaskLoader().load_tasks_from_dir(tmp_dir.name)
        self.assertDictEqual(result, {})
        tmp_dir.cleanup()

    def test_load_from_dir_multiple_python_files(self):
        tmp_dir = tempfile.TemporaryDirectory()
        tmp_file1 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir.name, delete=False)
        tmp_file2 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir.name, delete=False)
        tmp_file1.write(self.get_base_task_import_code_string())
        tmp_file2.write(self.get_base_task_import_code_string())
        for tasks_class_strings in self.get_custom_task_files_values()[:1]:
            tmp_file1.write(tasks_class_strings)
        tmp_file1.flush()
        for tasks_class_strings in self.get_custom_task_files_values()[1:]:
            tmp_file2.write(tasks_class_strings)
        tmp_file2.flush()
        importlib.invalidate_caches()
        result = TaskLoader().load_tasks_from_dir(tmp_dir.name)
        for task_name in ["my_task_one", "my_task_two", "my_task_three"]:
            self.assertIn(task_name, result)
            self.assertTrue(issubclass(result[task_name], BaseTask))
        tmp_dir.cleanup()

    def test_load_tasks_non_iterable_argument(self):
        with self.assertRaises(GOSTaskException):
            TaskLoader().load_tasks(5)

    def test_load_tasks_only_python_files(self):
        tmp_file1 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py")
        tmp_file2 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py")
        tmp_file1.write(self.get_base_task_import_code_string())
        tmp_file2.write(self.get_base_task_import_code_string())
        for tasks_class_strings in self.get_custom_task_files_values()[:1]:
            tmp_file1.write(tasks_class_strings)
        tmp_file1.flush()
        for tasks_class_strings in self.get_custom_task_files_values()[1:]:
            tmp_file2.write(tasks_class_strings)
        tmp_file2.flush()
        importlib.invalidate_caches()
        paths = [tmp_file1.name, tmp_file2.name]
        result = TaskLoader().load_tasks(paths=paths)
        for task_name in ["my_task_one", "my_task_two", "my_task_three"]:
            self.assertIn(task_name, result)
            self.assertTrue(issubclass(result[task_name], BaseTask))

    def test_load_tasks_non_string_value_in_paths(self):
        paths = ["path1", "path2", BaseTask(), "path3"]
        with self.assertRaises(GOSTaskException):
            TaskLoader().load_tasks(paths=paths)

    def test_load_tasks_only_files_some_files_are_non_python(self):
        tmp_file1 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py")
        tmp_file2 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py")
        tmp_file3 = tempfile.NamedTemporaryFile(mode="wt", suffix=".non_py")
        tmp_file1.write(self.get_base_task_import_code_string())
        tmp_file2.write(self.get_base_task_import_code_string())
        tmp_file3.write(self.get_base_task_import_code_string())
        for tasks_class_strings in self.get_custom_task_files_values()[:1]:
            tmp_file1.write(tasks_class_strings)
        tmp_file1.flush()
        for tasks_class_strings in self.get_custom_task_files_values()[1:2]:
            tmp_file2.write(tasks_class_strings)
        tmp_file2.flush()
        for tasks_class_strings in self.get_custom_task_files_values()[2:]:
            tmp_file3.write(tasks_class_strings)
        tmp_file3.flush()
        importlib.invalidate_caches()
        paths = [tmp_file1.name, tmp_file2.name, tmp_file3.name]
        result = TaskLoader().load_tasks(paths=paths)
        for task_name in ["my_task_one", "my_task_two"]:
            self.assertIn(task_name, result)
            self.assertTrue(issubclass(result[task_name], BaseTask))
        self.assertNotIn("my_task_three", result)

    def test_load_tasks_files_and_dirs(self):
        tmp_dir1 = tempfile.TemporaryDirectory()
        tmp_dir2 = tempfile.TemporaryDirectory()

        tmp_file1 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir1.name, delete=False)
        tmp_file2 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir1.name, delete=False)
        tmp_file3 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir1.name, delete=False)

        tmp_file4 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir2.name, delete=False)
        tmp_file5 = tempfile.NamedTemporaryFile(mode="wt", suffix=".non_py", dir=tmp_dir2.name, delete=False)

        tmp_file6 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py")
        tmp_file7 = tempfile.NamedTemporaryFile(mode="wt", suffix=".non_py")

        tmp_files = [tmp_file1, tmp_file2, tmp_file3, tmp_file4, tmp_file5, tmp_file6, tmp_file7]

        for cnt, tmp_file in enumerate(tmp_files):
            tmp_file.write(self.get_base_task_import_code_string())
            tmp_file.write(self.get_custom_task_files_values()[cnt])
            tmp_file.flush()
        importlib.invalidate_caches()
        paths = [tmp_dir1.name, tmp_dir2.name, tmp_file6.name, tmp_file7.name]
        result = TaskLoader().load_tasks(paths=paths)
        for task_name in ["my_task_one", "my_task_two", "my_task_three", "my_task_four", "my_task_six"]:
            self.assertIn(task_name, result)
            self.assertTrue(issubclass(result[task_name], BaseTask))
        self.assertNotIn("my_task_five", result)
        self.assertNotIn("my_task_seven", result)
        tmp_dir1.cleanup()
        tmp_dir2.cleanup()

    def test_load_tasks_from_dir_TaskException_propagation(self):
        tmp_dir = self._prepare_for_TaskException_propagation_test()
        with self.assertRaises(GOSTaskException):
            TaskLoader().load_tasks_from_dir(dir_path=tmp_dir.name, propagate_exceptions=True)
        tmp_dir.cleanup()

    def _prepare_for_TaskException_propagation_test(self):
        tmp_dir = tempfile.TemporaryDirectory()
        tmp_file1 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir.name, delete=False)
        tmp_file2 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", dir=tmp_dir.name, delete=False)
        tmp_file1.write(self.get_base_task_import_code_string())
        tmp_file1.write(self.get_custom_task_files_values()[0])
        tmp_file1.flush()
        tmp_file2.write(self.get_base_task_import_code_string())
        tmp_file2.write("""class Task(BaseTask):\n\tdef run(self, assembler_manager):\n\t\tpass\n""")
        tmp_file2.flush()
        importlib.invalidate_caches()
        return tmp_dir

    def test_load_tasks_from_dir_no_TaskException_propagation(self):
        tmp_dir = self._prepare_for_TaskException_propagation_test()
        result = TaskLoader().load_tasks_from_dir(dir_path=tmp_dir.name, propagate_exceptions=False)
        self.assertIn("my_task_one", result)
        tmp_dir.cleanup()

    def test_load_tasks_TaskException_propagation(self):
        tmp_dir = self._prepare_for_TaskException_propagation_test()
        tmp_file3 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", delete=True)
        tmp_file3.write(self.get_base_task_import_code_string())
        tmp_file3.write(self.get_custom_task_files_values()[1])
        tmp_file3.flush()
        importlib.invalidate_caches()
        with self.assertRaises(GOSTaskException):
            TaskLoader().load_tasks(paths=[tmp_dir.name, tmp_file3.name], propagate_exception=True)
        tmp_dir.cleanup()

    def test_load_tasks_no_TaskException_propagation(self):
        tmp_dir = self._prepare_for_TaskException_propagation_test()
        tmp_file3 = tempfile.NamedTemporaryFile(mode="wt", suffix=".py", delete=True)
        tmp_file3.write(self.get_base_task_import_code_string())
        tmp_file3.write(self.get_custom_task_files_values()[1])
        tmp_file3.flush()
        importlib.invalidate_caches()
        result = TaskLoader().load_tasks(paths=[tmp_dir.name, tmp_file3.name], propagate_exception=False)
        self.assertEqual(len(result), 3)  # base + two valid tasks
        self.assertIn("my_task_one", result)
        self.assertIn("my_task_two", result)
        tmp_dir.cleanup()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
__author__ = "Sergey Aganezov"
__email__ = "aganezov(at)gwu.edu"
__status__ = "develop"

import unittest

from teamcity import is_running_under_teamcity
from teamcity.unittestpy import TeamcityTestRunner

if __name__ == '__main__':
    all_tests = unittest.TestLoader().discover('./', pattern='test_*.py')
    if is_running_under_teamcity():
        runner = TeamcityTestRunner()
    else:
        runner = unittest.TextTestRunner()
    runner.run(all_tests)



# -*- coding: utf-8 -*-
from gos.configuration import Configuration
from gos.exceptions import GOSTaskException, GOSExecutableContainerException
from gos.executable_containers import ExecutableContainer
from gos.tasks import TaskLoader


class Manager(object):
    def __init__(self, config):
        self.configuration = config
        self.tasks_classes = {}
        self.tasks_instances = {}
        self.executable_containers_classes = {}
        self.executable_containers_instances = {}

    def initiate_tasks(self):
        """ Loads all tasks using `TaskLoader` from respective configuration option """
        self.tasks_classes = TaskLoader().load_tasks(
            paths=self.configuration[Configuration.ALGORITHM][Configuration.TASKS][Configuration.PATHS])

    def instantiate_tasks(self):
        """ All loaded tasks are initialized. Depending on configuration fails in such instantiations may be silent """
        self.tasks_instances = {}
        for task_name, task_class in self.tasks_classes.items():
            try:
                self.tasks_instances[task_name] = task_class()
            except Exception as ex:
                if not self.configuration[Configuration.ALGORITHM][Configuration.IOSF]:
                    raise GOSTaskException("An exception happened during the task instantiation."
                                           "{exception}".format(exception=ex))

    def initiate_executable_containers(self):
        for entry in self.configuration[Configuration.ALGORITHM]["executable_containers"]:
            if "reference" in entry:
                reference = entry["reference"]
                for ec_config in self.configuration[Configuration.ALGORITHM][reference]:
                    ec_config["group_reference_name"] = reference
                    result = ExecutableContainer.setup_from_config(manager=self, config=ec_config)
                    self.executable_containers_instances[result.name] = result
            elif "paths" in entry:
                paths = entry["paths"]
                for path in paths:
                    try:
                        for ec_instance in ExecutableContainer.setup_from_file(file_path=path):
                            self.executable_containers_instances[ec_instance.name] = ec_instance
                    except GOSExecutableContainerException:
                        continue
        if "pipeline" not in self.configuration[Configuration.ALGORITHM]["executable_containers"]:
            pipeline_config = self.configuration[Configuration.ALGORITHM]["pipeline"]
            if "name" not in pipeline_config:
                pipeline_config["name"] = "pipeline"
            self.executable_containers_instances["pipeline"] = ExecutableContainer.setup_from_config(manager=self,
                                                                                                     config=pipeline_config)

    def instantiate_executable_containers(self):
        for executable_container in self.executable_containers_instances.values():
            for entry_name in executable_container.entries_names:
                try:
                    entry = self.tasks_instances[entry_name]
                except KeyError:
                    entry = self.executable_containers_instances[entry_name]
                executable_container.entries.append(entry)

    def run(self):
        self.executable_containers_instances["pipeline"].run(manager=self)

    def get_task_instance(self, task_name):
        return self.tasks_instances[task_name]

    def get_executable_container_instance(self, ec_name):
        return self.executable_containers_instances[ec_name]



# -*- coding: utf-8 -*-
import unittest

import importlib
import tempfile
from gos.configuration import Configuration
from gos.exceptions import GOSTaskException
from gos.manager import Manager
from gos.tasks import BaseTask, TaskLoader
from tests.test_tasks import TaskLoaderTestCase


class ManagerTestCase(unittest.TestCase):
    def setUp(self):
        self.am = Manager(config=Configuration())

    def test_manager_init(self):
        config = Configuration()
        am = Manager(config=config)
        self.assertDictEqual(config, am.configuration)

    def create_correct_temporary_tasks_files(self):
        result = []
        tltc = TaskLoaderTestCase()
        for task_string_data in tltc.get_custom_task_files_values():
            tmp_file = tempfile.NamedTemporaryFile(mode="wt", suffix=".py")
            tmp_file.write(tltc.get_base_task_import_code_string())
            tmp_file.write(task_string_data)
            tmp_file.flush()
            result.append(tmp_file)
        return result

    def test_manager_initiate_tasks(self):
        tmp_files = self.create_correct_temporary_tasks_files()  # have to keep reference to tmp_file objects,
        # otherwise object are deleted by garbage collector and
        # corresponding files are deleted
        self.am.configuration[Configuration.ALGORITHM][Configuration.TASKS] = {
            Configuration.PATHS: [f.name for f in tmp_files]
        }
        importlib.invalidate_caches()
        self.am.initiate_tasks()
        self.assertTrue(hasattr(self.am, "tasks_classes"))
        self.assertTrue(isinstance(self.am.tasks_classes, dict))
        for task_name in TaskLoaderTestCase.get_tasks_names():
            self.assertIn(task_name, self.am.tasks_classes)
            self.assertTrue(issubclass(self.am.tasks_classes[task_name], BaseTask))

    def test_manager_instantiate_tasks(self):
        self.am.tasks_classes = self.get_tasks_classes()
        self.am.instantiate_tasks()
        self.check_task_instantiation_results()

    def get_tasks_classes(self):
        tmp_files = self.create_correct_temporary_tasks_files()
        paths = [f.name for f in tmp_files]
        importlib.invalidate_caches()
        return TaskLoader().load_tasks(paths=paths)

    def test_manager_instantiate_tasks_error_task_instantiation_no_silent_fail(self):
        self._prepare_error_task_instantiation_test_case()
        with self.assertRaises(GOSTaskException):
            self.am.instantiate_tasks()

    def test_manager_instantiate_tasks_error_task_instantiation_silent_fail(self):
        self._prepare_error_task_instantiation_test_case()
        self.am.configuration[Configuration.ALGORITHM][Configuration.IOSF] = True
        self.am.instantiate_tasks()
        del self.am.tasks_classes[self._get_task_class_with_error().name]
        self.check_task_instantiation_results()

    def _prepare_error_task_instantiation_test_case(self):
        self.am.tasks_classes = self.get_tasks_classes()
        self.am.configuration[Configuration.ALGORITHM][Configuration.IOSF] = False
        error_task = self._get_task_class_with_error()
        self.am.tasks_classes.update({error_task.name: error_task})

    def check_task_instantiation_results(self):
        for task_name in self.am.tasks_classes:
            self.assertIn(task_name, self.am.tasks_instances)
            self.assertIsInstance(self.am.tasks_instances[task_name], BaseTask)
            self.assertIsInstance(self.am.tasks_instances[task_name], self.am.tasks_classes[task_name])

    def _get_task_class_with_error(self):
        class ErrorTask(BaseTask):
            name = "error_task"

            def __init__(self):
                self.a = [1, 2, 3][3]

            def run(self, manager):
                pass
        return ErrorTask

    def test_manager_get_task(self):
        task = self._get_my_task_instance()
        self.am.tasks_instances[task.name] = task
        self.assertEqual(self.am.get_task_instance(task.name), task)

    def test_manager_get_task_name_does_not_exist(self):
        with self.assertRaises(KeyError):
            self.am.get_task_instance("non_existing_name")

    def _get_my_task_class(self):
        class MyTask(BaseTask):
            name = "my_task"

            def run(self, manager):
                pass

        return MyTask

    def _get_my_task_instance(self):
        return self._get_my_task_class()()


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from copy import deepcopy
from gos.configuration import Configuration


class ConfigurationTestCase(unittest.TestCase):
    def setUp(self):
        self.init_config = Configuration()

    def test_initialization_top_level(self):
        """ in simple initialization the top level section must be properly configured """
        config = Configuration()
        self.assertIn(config.DIR, config)
        self.assertIn(config.LOGGER, config)
        self.assertIn(config.IOSF, config)
        self.assertIn(config.INPUT, config)
        self.assertIn(config.ALGORITHM, config)
        self.assertIn(config.OUTPUT, config)

        self.assertIsInstance(config[config.LOGGER], dict)
        self.assertIsInstance(config[config.INPUT], dict)
        self.assertIsInstance(config[config.ALGORITHM], dict)
        self.assertIsInstance(config[config.OUTPUT], dict)

    def test_initialization_input_section(self):
        """ input section of the overall configuration must have some default init values and is predefined with them """
        config = Configuration()
        input_section = config[config.INPUT]
        self.assertIn(config.DIR, input_section)
        self.assertIn(config.LOGGER, input_section)
        self.assertIn(config.IOSF, input_section)
        self.assertIn(config.SOURCE, input_section)

        self.assertIsInstance(input_section[config.SOURCE], list)
        self.assertIsInstance(input_section[config.LOGGER], dict)

    def test_initialization_logger_section(self):
        """ logger section is a top level configuration for GOS wide logger """
        config = Configuration()
        logger_section = config[config.LOGGER]
        self.assertIn(config.NAME, logger_section)
        self.assertIn(config.LEVEL, logger_section)
        self.assertIn(config.FORMAT, logger_section)
        self.assertIn(config.DESTINATION, logger_section)

    def test_initialization_output_section(self):
        """ output section configuration for GOS results to be put in"""
        config = Configuration()
        output_section = config[config.OUTPUT]
        self.assertIn(config.DIR, output_section)
        self.assertIn(config.LOGGER, output_section)
        self.assertIn(config.IOSF, output_section)
        self.assertIn(config.ASSEMBLY_POINTS, output_section)
        self.assertIn(config.GENOMES, output_section)
        self.assertIn(config.STATS, output_section)

        self.assertIsInstance(output_section[config.STATS], dict)
        self.assertIsInstance(output_section[config.ASSEMBLY_POINTS], dict)
        self.assertIsInstance(output_section[config.GENOMES], dict)

    def test_initialization_algorithm_section_executable_containers(self):
        config = Configuration()
        algorithm_section = config[config.ALGORITHM]
        self.assertIn(config.EXECUTABLE_CONTAINERS, algorithm_section)

    def test_initialization_algorithm_section(self):
        """ algorithm section configuration for GOS workflow """
        config = Configuration()
        algorithm_section = config[config.ALGORITHM]
        self.assertIn(config.IOSF, algorithm_section)
        self.assertIn(config.LOGGER, algorithm_section)
        self.assertIn(config.TASKS, algorithm_section)
        self.assertIn(config.PIPELINE, algorithm_section)

        self.assertIsInstance(algorithm_section[config.TASKS], dict)
        self.assertIsInstance(algorithm_section[config.PIPELINE], dict)

    def test_update_with_default_top_level_dir_empty(self):
        """ top level configuration field "dir" default fallback when it is not specified """
        self.init_config[self.init_config.DIR] = None
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.DIR], os.getcwd())
        self.init_config[self.init_config.DIR] = ""
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.DIR], os.getcwd())

    def test_update_with_default_to_level_dir_predefined(self):
        """ top level configuration field "dir" default fallback when it is specified """
        self.init_config[self.init_config.DIR] = os.path.join("dir1", "dir2")
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.DIR], os.path.join("dir1", "dir2"))

    def test_update_with_default_top_level_io_silent_fail_empty(self):
        """ top level configuration field "io_silent_fail" default fallback when its not specified """
        self.init_config[self.init_config.IOSF] = None
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.IOSF], self.init_config.DEFAULT_IOSF)
        self.init_config[self.init_config.IOSF] = ""
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.IOSF], self.init_config.DEFAULT_IOSF)

    def test_update_with_default_top_level_io_silent_fail_predefined(self):
        """ top level configuration field "io_silent_fail" default fallback when its specified """
        self.init_config[self.init_config.IOSF] = True
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.IOSF], True)
        self.init_config[self.init_config.IOSF] = False
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.IOSF], False)
        self.init_config[self.init_config.IOSF] = "CustomValue"  # anything that works for if
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.IOSF], "CustomValue")

    def test_update_with_default_logger_name_empty(self):
        self.init_config[self.init_config.LOGGER][self.init_config.NAME] = ""
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.NAME],
                         self.init_config.DEFAULT_LOGGER_NAME)
        self.init_config[self.init_config.LOGGER][self.init_config.NAME] = None
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.NAME],
                         self.init_config.DEFAULT_LOGGER_NAME)

    def test_update_with_default_logger_name_predefined(self):
        self.init_config[self.init_config.LOGGER][self.init_config.NAME] = True
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.NAME],
                         str(True))
        self.init_config[self.init_config.LOGGER][self.init_config.NAME] = "MyName"
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.NAME],
                         "MyName")

    def test_update_with_default_logger_level_empty(self):
        self.init_config[self.init_config.LOGGER][self.init_config.LEVEL] = ""
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.LEVEL],
                         self.init_config.DEFAULT_LOGGER_LEVEL)
        self.init_config[self.init_config.LOGGER][self.init_config.LEVEL] = None
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.LEVEL],
                         self.init_config.DEFAULT_LOGGER_LEVEL)

    def test_update_with_default_logger_level_predefined(self):
        self.init_config[self.init_config.LOGGER][self.init_config.LEVEL] = "MyLevel"
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.LEVEL],
                         "MyLevel")
        self.init_config[self.init_config.LOGGER][self.init_config.LEVEL] = True
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.LEVEL],
                         str(True))

    def test_update_with_default_logger_format_empty(self):
        self.init_config[self.init_config.LOGGER][self.init_config.FORMAT] = ""
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.FORMAT],
                         self.init_config.DEFAULT_LOGGER_FORMAT)
        self.init_config[self.init_config.LOGGER][self.init_config.FORMAT] = None
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.FORMAT],
                         self.init_config.DEFAULT_LOGGER_FORMAT)

    def test_update_with_default_logger_format_predefined(self):
        self.init_config[self.init_config.LOGGER][self.init_config.FORMAT] = "MyFormat"
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.FORMAT],
                         "MyFormat")
        self.init_config[self.init_config.LOGGER][self.init_config.FORMAT] = True
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.LOGGER][self.init_config.FORMAT],
                         str(True))

    def test_update_with_default_input_source_empty(self):
        for empty_value in (None, ""):
            self.init_config[self.init_config.INPUT][self.init_config.SOURCE] = empty_value
            self.init_config.update_with_default_values()
            self.assertListEqual(self.init_config[self.init_config.INPUT][self.init_config.SOURCE],
                                 [])

    def test_update_with_default_input_source_predefined(self):
        for source_value in [["path1", "path2"], ["path3", "path4", "path5"]]:
            self.init_config[self.init_config.INPUT][self.init_config.SOURCE] = source_value
            self.init_config.update_with_default_values()
            self.assertListEqual(source_value,
                                 self.init_config[self.init_config.INPUT][self.init_config.SOURCE])

    def test_update_with_default_input_dir_empty(self):
        for empty_value in (None, ""):
            self.init_config[self.init_config.INPUT][self.init_config.DIR] = empty_value
            self.init_config.update_with_default_values()
            self.assertEqual(self.init_config[self.init_config.INPUT][self.init_config.DIR],
                             self.init_config.DEFAULT_INPUT_DIR)

    def test_update_with_default_input_io_silent_fail_empty(self):
        for empty_value in (None, ""):
            for top_level_iosf_value in (True, False):
                self.init_config[self.init_config.INPUT][self.init_config.IOSF] = empty_value
                self.init_config[self.init_config.IOSF] = top_level_iosf_value
                self.init_config.update_with_default_values()
                self.assertEqual(self.init_config[self.init_config.INPUT][self.init_config.IOSF],
                                 top_level_iosf_value)

    def get_list_of_logger_configurations(self):
        return [{
            self.init_config.NAME: "Logger Name 1",
            self.init_config.LEVEL: "info 1",
            self.init_config.FORMAT: "format 1",
            self.init_config.DESTINATION: "destination 1"
        }, {
            self.init_config.NAME: "Logger Name 2",
            self.init_config.LEVEL: "info 2",
            self.init_config.FORMAT: "format 2",
            self.init_config.DESTINATION: "destination 2"
        }]

    def test_update_with_default_input_logger_empty(self):
        top_level_loggers = self.get_list_of_logger_configurations()
        for logger_config in top_level_loggers:
            self.init_config = Configuration()
            self.init_config[self.init_config.INPUT][self.init_config.LOGGER] = {}
            self.init_config[self.init_config.LOGGER] = logger_config
            self.init_config.update_with_default_values()
            self.assertDictEqual(self.init_config[self.init_config.INPUT][self.init_config.LOGGER],
                                 logger_config)

    def test_update_with_default_input_logger_partially_predefined(self):
        partial_logger_configs = [
            {self.init_config.NAME: "My name",
             self.init_config.LEVEL: "My level"},
            {self.init_config.LEVEL: "My level 2"},
            {self.init_config.FORMAT: "My format",
             self.init_config.DESTINATION: "My destination"}
        ]
        for partial_logger_config in partial_logger_configs:
            for full_logger_config in self.get_list_of_logger_configurations():
                self.init_config[self.init_config.INPUT][self.init_config.LOGGER] = deepcopy(partial_logger_config)
                self.init_config[self.init_config.LOGGER] = full_logger_config
                self.init_config.update_with_default_values()
                for key, value in full_logger_config.items():
                    if key not in partial_logger_config:
                        self.assertEqual(full_logger_config[key],
                                         self.init_config[self.init_config.INPUT][self.init_config.LOGGER][key])
                    else:
                        self.assertEqual(partial_logger_config[key],
                                         self.init_config[self.init_config.INPUT][self.init_config.LOGGER][key])

    def test_update_with_default_input_logger_specified(self):
        for full_logger_spec in self.get_list_of_logger_configurations():
            self.init_config[self.init_config.INPUT][self.init_config.LOGGER] = deepcopy(full_logger_spec)
            self.init_config.update_with_default_values()
            self.assertDictEqual(full_logger_spec,
                                 self.init_config[self.init_config.INPUT][self.init_config.LOGGER])

    def test_update_with_default_output_dir_empty(self):
        for empty_value in (None, ""):
            self.init_config[self.init_config.OUTPUT][self.init_config.DIR] = empty_value
            self.init_config.update_with_default_values()
            self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.DIR],
                             os.path.join(self.init_config[self.init_config.DIR],
                                          self.init_config.DEFAULT_OUTPUT_DIR))

    def test_update_with_default_output_io_silent_fail_empty(self):
        for empty_value in (None, ""):
            for top_level_iosf_value in (True, False):
                self.init_config[self.init_config.OUTPUT][self.init_config.IOSF] = empty_value
                self.init_config[self.init_config.IOSF] = top_level_iosf_value
                self.init_config.update_with_default_values()
                self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.IOSF],
                                 top_level_iosf_value)

    def tet_update_with_default_output_logger_empty(self):
        for empty_value in (None, "", {}):
            top_level_loggers = self.get_list_of_logger_configurations()
            for logger_config in top_level_loggers:
                self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER] = empty_value
                self.init_config[self.init_config.LOGGER] = logger_config
                self.init_config.update_with_default_values()
                self.assertDictEqual(self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER],
                                     logger_config)

    def test_update_with_default_output_logger_partially_predefined(self):
        partial_logger_configs = [
            {self.init_config.NAME: "My name",
             self.init_config.LEVEL: "My level"},
            {self.init_config.LEVEL: "My level 2"},
            {self.init_config.FORMAT: "My format",
             self.init_config.DESTINATION: "My destination"}
        ]
        for partial_logger_config in partial_logger_configs:
            for full_logger_config in self.get_list_of_logger_configurations():
                self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER] = deepcopy(partial_logger_config)
                self.init_config[self.init_config.LOGGER] = full_logger_config
                self.init_config.update_with_default_values()
                for key, value in full_logger_config.items():
                    if key not in partial_logger_config:
                        self.assertEqual(full_logger_config[key],
                                         self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER][key])
                    else:
                        self.assertEqual(partial_logger_config[key],
                                         self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER][key])

    def test_update_with_default_output_logger_specified(self):
        for full_logger_spec in self.get_list_of_logger_configurations():
            self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER] = deepcopy(full_logger_spec)
            self.init_config.update_with_default_values()
            self.assertDictEqual(full_logger_spec,
                                 self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER])

    def test_update_with_default_output_stats_empty(self):
        for dir_name in ("output_dir1", "output_dir2", "output_dir3"):
            for iosf_value in (True, False):
                for logger_value in self.get_list_of_logger_configurations():
                    self.init_config[self.init_config.OUTPUT][self.init_config.STATS] = {}
                    self.init_config[self.init_config.OUTPUT][self.init_config.DIR] = dir_name
                    self.init_config[self.init_config.OUTPUT][self.init_config.IOSF] = iosf_value
                    self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER] = logger_value
                    self.init_config.update_with_default_values()
                    self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.STATS][self.init_config.FILE],
                                     self.init_config.DEFAULT_OUTPUT_STATS_FILE)
                    self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.STATS][self.init_config.DIR],
                                     self.init_config.DEFAULT_OUTPUT_STATS_DIR)
                    self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.STATS][self.init_config.IOSF],
                                     iosf_value)
                    self.assertDictEqual(self.init_config[self.init_config.OUTPUT][self.init_config.STATS][self.init_config.LOGGER],
                                         logger_value)
                    self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.STATS][self.init_config.FILE],
                                     self.init_config.DEFAULT_OUTPUT_STATS_FILE)

    def get_full_stats_configs(self):
        return [
            {self.init_config.DIR: "stat_dir_predefined_1",
             self.init_config.FILE: "file_predefined_1.txt",
             self.init_config.LOGGER: self.get_list_of_logger_configurations()[0],
             self.init_config.IOSF: True},
            {self.init_config.DIR: "stat_dir_predefined_2",
             self.init_config.FILE: "file_predefined_2.txt",
             self.init_config.LOGGER: self.get_list_of_logger_configurations()[0],
             self.init_config.IOSF: False},
        ]

    def test_update_with_default_output_stats_partially_predefined(self):
        partial_stats_configs = [
            {self.init_config.DIR: "stats_dir",
             self.init_config.FILE: "my_file_name.txt"},
            {self.init_config.IOSF: True},
            {self.init_config.DIR: "my_dir",
             self.init_config.FILE: "my_file_name2.txt",
             self.init_config.LOGGER: self.get_list_of_logger_configurations()[0]}
        ]
        for partial_stats_config in partial_stats_configs:
            for full_stats_config in self.get_full_stats_configs():
                self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER] = full_stats_config[self.init_config.LOGGER]
                self.init_config[self.init_config.OUTPUT][self.init_config.STATS] = deepcopy(partial_stats_config)
                self.init_config[self.init_config.OUTPUT][self.init_config.IOSF] = full_stats_config[self.init_config.IOSF]
                self.init_config.update_with_default_values()
                for key, value in partial_stats_config.items():
                    self.assertEqual(partial_stats_config[key],
                                     self.init_config[self.init_config.OUTPUT][self.init_config.STATS][key])

    def test_update_with_default_output_stats_predefined(self):
        for full_stats_config in self.get_full_stats_configs():
            self.init_config[self.init_config.OUTPUT][self.init_config.STATS] = deepcopy(full_stats_config)
            self.init_config.update_with_default_values()
            self.assertDictEqual(self.init_config[self.init_config.OUTPUT][self.init_config.STATS],
                                 full_stats_config)

    def test_update_with_default_output_assembly_points_empty(self):
        self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS] = {}
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.FILE],
                         self.init_config.DEFAULT_OUTPUT_AP_FILE)
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.DIR],
                         self.init_config.DEFAULT_OUTPUT_AP_DIR)
        self.assertDictEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.LOGGER],
                             self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER])
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.GENOME_SPECIFIC],
                         self.init_config.DEFAULT_OUTPUT_AP_GENOME_SPECIFIC)
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.GENOME_SPECIFIC_FNP],
                         self.init_config.DEFAULT_OUTPUT_AP_GSFNP)
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.IOSF],
                         self.init_config[self.init_config.OUTPUT][self.init_config.IOSF])

    def test_update_with_default_output_assembly_points_partially_predefined(self):
        partial_ap_config = {
            self.init_config.DIR: "my_ap_dir",
            self.init_config.GENOME_SPECIFIC: True,
            self.init_config.IOSF: False
        }
        self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS] = partial_ap_config
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.FILE],
                         self.init_config.DEFAULT_OUTPUT_AP_FILE)
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.DIR],
                         partial_ap_config[self.init_config.DIR])
        self.assertDictEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.LOGGER],
                             self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER])
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.GENOME_SPECIFIC],
                         partial_ap_config[self.init_config.GENOME_SPECIFIC])
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.GENOME_SPECIFIC_FNP],
                         self.init_config.DEFAULT_OUTPUT_AP_GSFNP)
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][self.init_config.IOSF],
                         partial_ap_config[self.init_config.IOSF])

    def test_update_with_default_output_assembly_points_predefined(self):
        full_ap_config = {
            self.init_config.FILE: "my_ap_file.txt",
            self.init_config.DIR: "my_ap_dir",
            self.init_config.IOSF: True,
            self.init_config.GENOME_SPECIFIC: True,
            self.init_config.GENOME_SPECIFIC_FNP: "my_patter_string_{genome_name}.txt",
            self.init_config.LOGGER: self.get_list_of_logger_configurations()[0]
        }
        self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS] = full_ap_config
        self.init_config.update_with_default_values()
        for key, value in full_ap_config.items():
            self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.ASSEMBLY_POINTS][key],
                             full_ap_config[key])

    def test_update_with_default_output_genomes_empty(self):
        self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES] = {}
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES][self.init_config.DIR],
                         self.init_config.DEFAULT_OUTPUT_GENOMES_DIR)
        self.assertDictEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES][self.init_config.LOGGER],
                             self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER])
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES][self.init_config.IOSF],
                         self.init_config[self.init_config.OUTPUT][self.init_config.IOSF])
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES][self.init_config.OUTPUT_NG_FRAGMENTS],
                         self.init_config.DEFAULT_OUTPUT_GENOMES_ONGF)

    def test_update_with_default_output_genomes_partially_predefined(self):
        partial_genomes_config = {
            self.init_config.OUTPUT_NG_FRAGMENTS: True,
            self.init_config.IOSF: False
        }
        self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES] = partial_genomes_config
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES][self.init_config.DIR],
                         self.init_config.DEFAULT_OUTPUT_GENOMES_DIR)
        self.assertDictEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES][self.init_config.LOGGER],
                             self.init_config[self.init_config.OUTPUT][self.init_config.LOGGER])
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES][self.init_config.IOSF],
                         partial_genomes_config[self.init_config.IOSF])
        self.assertEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES][self.init_config.OUTPUT_NG_FRAGMENTS],
                         partial_genomes_config[self.init_config.OUTPUT_NG_FRAGMENTS])

    def test_update_with_default_output_genomes_predefined(self):
        predefined_genome_config = {
            self.init_config.OUTPUT_NG_FRAGMENTS: True,
            self.init_config.IOSF: False,
            self.init_config.LOGGER: self.get_list_of_logger_configurations()[0],
            self.init_config.DIR: "my_genome_dir"
        }
        self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES] = predefined_genome_config
        self.init_config.update_with_default_values()
        self.assertDictEqual(self.init_config[self.init_config.OUTPUT][self.init_config.GENOMES],
                             predefined_genome_config)

    def test_update_with_default_algorithm_empty(self):
        self.init_config[self.init_config.ALGORITHM] = {}
        self.init_config.update_with_default_values()
        self.assertDictEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.LOGGER],
                             self.init_config[self.init_config.LOGGER])
        self.assertDictEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.TASKS], {
            self.init_config.PATHS: [self.init_config.DEFAULT_ALGORITHM_TASKS_PATH]})
        self.assertEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.EXECUTABLE_CONTAINERS], [])
        expected_pipeline_config = {
            self.init_config.LOGGER: self.init_config[self.init_config.ALGORITHM][self.init_config.LOGGER],
            self.init_config.SELF_LOOP: self.init_config.DEFAULT_ALGORITHM_PIPELINE_SELF_LOOP,
            self.init_config.ENTRIES: [],
            self.init_config.IOSF: self.init_config[self.init_config.ALGORITHM][self.init_config.IOSF]
        }
        self.assertDictEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.PIPELINE],
                             expected_pipeline_config)

    def test_update_with_default_algorithm_predefined_tasks_paths(self):
        my_path_list = ["my_path1", "my_path2"]
        self.init_config[self.init_config.ALGORITHM] = {
            self.init_config.TASKS: {
                self.init_config.PATHS: deepcopy(my_path_list)
            }
        }
        self.init_config.update_with_default_values()
        self.assertIn(self.init_config.DEFAULT_ALGORITHM_TASKS_PATH,
                      self.init_config[self.init_config.ALGORITHM][self.init_config.TASKS][self.init_config.PATHS])
        for my_path in my_path_list:
            self.assertIn(my_path,
                          self.init_config[self.init_config.ALGORITHM][self.init_config.TASKS][self.init_config.PATHS])

    def test_update_with_default_algorithm_pipeline_logger(self):
        self.init_config[self.init_config.ALGORITHM] = {
            self.init_config.PIPELINE: {
                self.init_config.ENTRIES: []
            }
        }
        self.init_config.update_with_default_values()
        self.assertDictEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.PIPELINE][self.init_config.LOGGER],
                             self.init_config[self.init_config.ALGORITHM][self.init_config.LOGGER])

    def test_update_with_default_algorithm_predefined(self):
        predefined_algorithm_config = {
            self.init_config.IOSF: False,
            self.init_config.LOGGER: self.get_list_of_logger_configurations()[0],
            self.init_config.TASKS: {
                self.init_config.PATHS: ["my_path_1", "my_path_2"]
            },
            self.init_config.PIPELINE: {
                self.init_config.LOGGER: self.get_list_of_logger_configurations()[1],
                self.init_config.SELF_LOOP: False,
                self.init_config.ENTRIES: ["round1", "round2"]
            }
        }
        self.init_config[self.init_config.ALGORITHM] = deepcopy(predefined_algorithm_config)
        self.init_config.update_with_default_values()
        self.assertEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.IOSF],
                         predefined_algorithm_config[self.init_config.IOSF])
        self.assertDictEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.LOGGER],
                             predefined_algorithm_config[self.init_config.LOGGER])
        self.assertListEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.TASKS][self.init_config.PATHS],
                             [self.init_config.DEFAULT_ALGORITHM_TASKS_PATH] +
                             predefined_algorithm_config[self.init_config.TASKS][self.init_config.PATHS])
        predefined_algorithm_config[self.init_config.PIPELINE][self.init_config.IOSF] = self.init_config[self.init_config.ALGORITHM][self.init_config.IOSF]
        self.assertDictEqual(self.init_config[self.init_config.ALGORITHM][self.init_config.PIPELINE],
                             predefined_algorithm_config[self.init_config.PIPELINE])

    def test_update_with_default_algorithm_specified_executable_container_instantiation(self):
        self.set_up_executable_containers_for_algorithm_section()
        self.init_config.update_with_default_values()
        self.assertIsInstance(self.init_config[self.init_config.ALGORITHM]["stages"], list)

    def set_up_executable_containers_for_algorithm_section(self):
        ecs = [
            {
                "name": "stage",
                "reference": "stages",
                "entry_type_name": "task"
            }
        ]
        self.init_config[self.init_config.ALGORITHM][self.init_config.EXECUTABLE_CONTAINERS] = ecs

    def test_update_with_default_algorithm_automatically_generated_reference_for_executable_container(self):
        self.init_config[self.init_config.ALGORITHM][self.init_config.EXECUTABLE_CONTAINERS] = [{
            "name": "stage",
            "entry_type_name": "task"
        }]
        self.init_config.update_with_default_values()
        ecs = self.init_config[self.init_config.ALGORITHM][self.init_config.EXECUTABLE_CONTAINERS]
        self.assertEqual(ecs[0]["reference"], "stages")

    def test_update_with_default_algorithm_specified_executable_container_partially_specification(self):
        self.set_up_executable_containers_for_algorithm_section()
        self.init_config[self.init_config.ALGORITHM]["stages"] = [
            {
                self.init_config.NAME: "stage1",
            },
            {
                self.init_config.NAME: "stage2",
                self.init_config.ENTRIES: ["task1", "task2"]
            },
            {
                self.init_config.NAME: "stage3",
                self.init_config.SELF_LOOP: False,
                self.init_config.ENTRIES: ["task1", "task2", "task3"]
            }
        ]
        self.init_config.update_with_default_values()
        stages = self.init_config[self.init_config.ALGORITHM]["stages"]
        self.assertIsInstance(stages, list)
        self.assertEqual(len(stages), 3)
        stage1, stage2, stage3 = stages
        self.assertEqual(stage1[self.init_config.NAME], "stage1")
        self.assertEqual(stage1[self.init_config.SELF_LOOP], self.init_config.DEFAULT_ALGORITHM_EC_SELF_LOOP)
        self.assertListEqual(stage1[self.init_config.ENTRIES], [])

        self.assertEqual(stage2[self.init_config.NAME], "stage2")
        self.assertEqual(stage2[self.init_config.SELF_LOOP], self.init_config.DEFAULT_ALGORITHM_EC_SELF_LOOP)
        self.assertListEqual(stage2[self.init_config.ENTRIES], ["task1", "task2"])

        self.assertEqual(stage3[self.init_config.NAME], "stage3")
        self.assertEqual(stage3[self.init_config.SELF_LOOP], False)
        self.assertListEqual(stage3[self.init_config.ENTRIES], ["task1", "task2", "task3"])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from gos.exceptions import GOSExecutableContainerException
from gos.utils.load import Loader

DEFAULT_SELF_LOOP = False


class ExecutableContainer(object):
    name = "executable_container"
    type_name = "executable_container"

    def __init__(self, name=None, type_name=None, group_reference_name=None, self_loop=DEFAULT_SELF_LOOP, do_self_loop=False,
                 entries_names=None, entries=None,
                 entries_type_names=None, logger=None):

        self.name = self.__class__.name if name is None else name
        self.type_name = self.__class__.type_name if type_name is None else type_name

        if group_reference_name is None:
            group_reference_name = self.group_reference_name if \
                hasattr(self, "group_reference_name") and self.group_reference_name is not None else \
                self._get_default_group_reference_name()
        self.group_reference_name = group_reference_name

        if entries_names is None:
            entries_names = self.entries_names if hasattr(self, "entries_names") else []
        self.entries_names = entries_names

        if entries_type_names is None:
            entries_type_names = self.entries_type_names if hasattr(self, "entries_type_names") else []
        self.entries_type_names = entries_type_names

        self.entries = [] if entries is None else entries

        self.self_loop = self_loop
        self.do_self_loop = do_self_loop
        self.logger = logger

    def _get_default_group_reference_name(self):
        return self.name + "s"

    def run(self, manager):
        for entry in self.entries:
            entry.do_self_loop = False
            entry.run(manager=manager)
            while entry.self_loop and entry.do_self_loop:
                entry.do_self_loop = False
                entry.run(manager=manager)

    @staticmethod
    def setup_from_config(manager, config):
        try:
            name = config["name"]
        except KeyError:
            raise GOSExecutableContainerException()
        reference = config.get("group_reference_name")
        entries_type_names = config.get("entries_type_names")
        entries_names = config.get("entries_names")
        self_loop = config.get("self_loop", DEFAULT_SELF_LOOP)
        result = ExecutableContainer(name=name, self_loop=self_loop, entries_names=entries_names, entries_type_names=entries_type_names,
                                     group_reference_name=reference)
        manager.logger.debug("Created {name} executable container from config".format(name=result.name))
        return result

    @staticmethod
    def setup_from_file(file_path):
        flag = False
        file_name, module_path, objects = Loader.import_custom_python_file(file_path)
        for entry in objects:
            try:
                if issubclass(entry, ExecutableContainer) and entry.__name__ != ExecutableContainer.__name__:
                    if entry.name == ExecutableContainer.name:
                        continue
                    elif not hasattr(entry, "setup"):
                        continue
                    flag = True
                    result = entry()
                    result.setup()
                    yield result
            except TypeError:
                continue
        if not flag:
            raise GOSExecutableContainerException()



# -*- coding: utf-8 -*-
__author__ = "aganezov"

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name="gos",
      version="0.1.3",
      packages=["gos", "tests", "gos.utils",
                "gos.algo", "gos.algo.executable_containers"],
      install_requires=list(map(lambda entry: entry.strip(), open("requirements.txt", "rt").readlines())),
      author="Sergey Aganezov",
      author_email="aganezov@gwu.edu",
      description="Generically organizable supervisor to create multi-level executable pipelines",
      license="LGPLv3",
      keywords=["breakpoint graph", "data structures", "python", "scaffolding", "gene order"],
      url="https://github.com/aganezov/gos",
      classifiers=[
          "Development Status :: 1 - Planning",
          "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5'
      ]
      )
